import "C:/Home/Repository/ModelicaCVS/Modelica_Fluid/package.mo";


package ThermodynamicTutorial "models for the Modelica 2005 tutorial" 
  import SI = Modelica.SIunits;
  package Components "example components for tutorial" 
    model BadOrifice "an orifice that works badly with back flow " 
      extends Modelica_Fluid.Interfaces.PartialTwoPortTransport;
    //  input Medium.Density d 
    //    "upstream density for detailed friction model used for pressure loss calculation";
      SI.Pressure dp "Pressure loss due to pipe friction";
      parameter SI.AbsolutePressure dp_nominal(min=1.e-10) = 1.0e5 
        " Nominal pressure drop";
      parameter SI.MassFlowRate m_flow_nominal(min=1.e-10) = 1 
        " Nominal mass flow rate at nominal pressure drop";
      parameter Real k = dp_nominal/m_flow_nominal^2 "friction loss parameter";
      annotation (Icon(Line(points=[-80,-80; -80,80; 80,-80; 80,80], style(
              color=0,
              rgbcolor={0,0,0},
              thickness=2))));
    equation 
       dp = port_a.p - port_b.p;
       m_flow = if dp > 0 then sqrt(dp/k) else -sqrt(dp/k);
    end BadOrifice;
    
    model HeatTransferVolume 
      "Fixed volume associated with a port by the finite volume method (used to build up physical components; fulfills mass and energy balance)" 
      import SI = Modelica.SIunits;
      
      replaceable package Medium = Modelica.Media.Interfaces.PartialMedium 
        "Medium model" 
         annotation (choicesAllMatching=true);
      
      parameter SI.Volume V=1e-6 "Fixed size of junction volume";
      parameter Boolean initializeVolume = true 
        "true if volume should use initial values for T and p";
      parameter Boolean use_p_start=true "select p_start or d_start" 
        annotation (Evaluate=true, Dialog(group="Initial pressure or initial density"));
      parameter Medium.AbsolutePressure p_start = Medium.reference_p 
        "Initial pressure" 
        annotation (Dialog(group="Initial pressure or initial density", enable=use_p_start));
      parameter Medium.Density d_start=1 "Initial density" 
        annotation (Dialog(group="Initial pressure or initial density", enable=not use_p_start));
      parameter Boolean use_T_start=true "select T_start or h_start" 
        annotation (Evaluate=true, Dialog(group="Initial temperature or initial specific enthalpy"));
      parameter Medium.Temperature T_start = Modelica.SIunits.Conversions.from_degC(20) 
        "Initial temperature" 
        annotation (Dialog(group="Initial temperature or initial specific enthalpy", enable=use_T_start));
      parameter Medium.SpecificEnthalpy h_start = 1.e4 
        "Initial specific enthalpy" 
        annotation (Dialog(group="Initial temperature or initial specific enthalpy", enable=not use_T_start));
      parameter Medium.MassFraction X_start[Medium.nX] = Medium.reference_X 
        "Initial mass fractions m_i/m" 
        annotation (Dialog(group="Only for multi-substance flow", enable=Medium.nX > 0));
      
      Modelica_Fluid.Interfaces.FluidPort_a flowport(redeclare package Medium 
          = Medium)                                       annotation (extent=[-20,
            -20; 20,20],                                                                         rotation=0);
      Medium.BaseProperties medium(preferredMediumStates=true,
                                   T(start = T_start),
                                   p(start = p_start),
                                   h(start = h_start),
                                   d(start = d_start),
                                   X(start = X_start));
      SI.Energy U "Internal energy of port volume";
      SI.Mass m "Mass of junction volume";
      SI.Mass mXi[Medium.nXi] "Independent substance masses of port volume";
      
      annotation (
       Icon(
          Text(extent=[-144, 178; 146, 116], string="%name"),
          Text(
            extent=[-130, -108; 144, -150],
            style(color=0),
            string="V=%V"),
          Rectangle(extent=[-100,80; 100,-80], style(
              color=0,
              rgbcolor={0,0,0},
              thickness=2,
              gradient=2,
              fillColor=71,
              rgbfillColor={85,170,255}))),
                             Documentation(info="<html>
<p>
This component models the <b>volume</b> of <b>fixed size</b> that is
associated with the <b>fluid port</b> to which it is connected.
This means that all medium properties inside the volume, are identical
to the port medium properties. In particular, the specific enthalpy
inside the volume (= medium.h) is always identical to the specific enthalpy
in the port (port.h = medium.h). Usually, this model is used when
discretizing a component according to the finite volume method into
volumes in internal ports that only store energy and mass and into
transport elements that just transport energy, mass and momentum
between the internal ports without storing these quantities during the
transport. This splitting is only possible under certain assumptions.
</p>
</html>"),
        Diagram);
      
      Modelica_Fluid.Interfaces.HeatPort_a heatport 
        annotation (extent=[-20,60; 20,100]);
    initial equation 
      if initializeVolume then
      if not Medium.singleState then
        if use_p_start then
           medium.p = p_start;
        else
           medium.d = d_start;
        end if;
      end if;
        
      if use_T_start then
         medium.T = T_start;
      else
         medium.h = h_start;
      end if;
      medium.Xi = X_start[1:Medium.nXi];
      end if;
    equation 
      // port = medium properties
        flowport.p = medium.p;
        flowport.h = medium.h;
        flowport.Xi = medium.Xi;
        heatport.T = medium.T;
      
      // Total quantities
         m    = V*medium.d;
         mXi = m*medium.Xi;
         U    = m*medium.u;
      
      // Mass and energy balance
         der(m)    = flowport.m_flow;
         der(mXi) = flowport.mXi_flow;
         der(U)   = flowport.H_flow + heatport.Q_flow;
    end HeatTransferVolume;
    
  protected 
    model GoodOrifice "an orifice that works badly with back flow " 
      extends Modelica_Fluid.Interfaces.PartialTwoPortTransport;
    //  input Medium.Density d 
    //    "upstream density for detailed friction model used for pressure loss calculation";
      SI.Pressure dp "Pressure loss due to pipe friction";
      parameter SI.Pressure p_small(min=1.e-10) = 10 
        "A small laminar region is introduced around p_small";
      parameter SI.AbsolutePressure dp_nominal(min=1.e-10) = 1.0e5 
        " Nominal pressure drop";
      parameter SI.MassFlowRate m_flow_nominal(min=1.e-10) = 1 
        " Nominal mass flow rate at nominal pressure drop";
      parameter Real C1=0.5/sqrt(p_small) - 3.0*C3*p_small^2 
        "Coefficient 1 of cubic polynomial in the laminar region";
      parameter Real C3=-0.25/(sqrt(p_small)*p_small^2) 
        "Coefficient 3 of cubic polynomial in the laminar region";
      parameter Real k = dp_nominal/m_flow_nominal^2 "friction loss parameter";
      annotation (Icon(Line(points=[-80,-80; -80,80; 80,-80; 80,80], style(
              color=0,
              rgbcolor={0,0,0},
              thickness=2))));
    equation 
       dp = port_a.p - port_b.p;
       m_flow = noEvent(if dp > p_small then sqrt(dp) else (if dp < -p_small then 
          -sqrt(-dp) else (C1 + C3*dp*dp)*dp))/sqrt(k);
    end GoodOrifice;
  public 

    partial model ValveBase "Base model for valves" 
     extends Modelica_Fluid.Interfaces.PartialTwoPortTransport;
     annotation (Icon(
          Line(points=[0, 40; 0, 0], style(
              color=0,
              thickness=2,
              fillPattern=1)),
          Polygon(points=[-80, 40; -80, -40; 0, 0; -80, 40], style(
              color=0,
              thickness=2,
              fillPattern=1)),
          Polygon(points=[80, 40; 0, 0; 80, -40; 80, 40], style(
              color=0,
              thickness=2,
              fillPattern=1)),
          Rectangle(extent=[-20, 60; 20, 40], style(
              color=0,
              fillColor=0,
              fillPattern=1))), Diagram,
        Documentation(revisions="<html>
<ul>
<li><i>27 Sep 2004</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted to Modelica_Fluid.</li>
<li><i>1 Jul 2004</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Valve models restructured using inheritance. <br>
       Adapted to Modelica.Media.</li>
<li><i>1 Oct 2003</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       First release.</li>
</ul>
</html>"));
      // replaceable package Medium = Modelica.Media.Interfaces.PartialMedium 
      //   "Medium model" 
      //                annotation(choicesAllMatching= true);
      // Medium.BaseProperties fluid;
      parameter Integer CvData "(0: Av | 1: Kv | 2: Cv | 3: OpPoint)";
      parameter Modelica.SIunits.Area Avnom=0 "Av (metric) flow coefficient";
      parameter Real Kvnom(unit="m^3/h")=0 "Kv (metric) flow coefficient";
      parameter Real Cvnom(unit="USG/min")=0 "Cv (US) flow coefficient";
      parameter Medium.AbsolutePressure dpnom "Nominal pressure drop";
      parameter Medium.MassFlowRate wnom=0 "Nominal mass flowrate";
      parameter Medium.Density rhonom=0 "Nominal inlet density";
      parameter Boolean CheckValve=false "Reverse flow stopped";
      parameter Real b=0.01 "Regularisation factor";
      replaceable function FlowChar = linear "Flow characteristic";
      Medium.MassFlowRate w "Mass flowrate";
      Real Av "Flow coefficient";
      Medium.Density rho "Inlet density";
      Medium.Temperature Tin "Inlet temperature";
      // Modelica_Fluid.Interfaces.FluidPort_a inlet(redeclare package Medium = 
      //       Medium) annotation (extent=[-120, -20; -80, 20]);
      // Modelica_Fluid.Interfaces.FluidPort_b outlet(redeclare package Medium = 
      //       Medium)  annotation (extent=[80, -20; 120, 20]);
      Modelica.Blocks.Interfaces.RealInput theta 
        annotation (extent=[-20, 60; 20, 100], rotation=-90);
      annotation (
        Icon(Text(extent=[-100, -40; 100, -80], string="%name")),
        Diagram,
        Documentation(info="<HTML>
<p>This is the base model for the <tt>ValveLiq</tt>, <tt>ValveLiqChoked</tt>, and <tt>ValveVap</tt> valve models. The model is based on the IEC 534 / ISA S.75 standards for valve sizing.
<p>The model optionally supports reverse flow conditions (assuming symmetrical behaviour) or check valve operation, and has been suitably modified to avoid numerical singularities at zero pressure drop.
<p>The flow characteristic can be customised.
<p><b>Modelling options</b></p>
<p>The following options are available to specify the valve flow coefficient in fully open conditions:
<ul><li><tt>CvData = 0</tt>: the flow coefficient is given by the metric Av coefficient <tt>Avnom</tt> (m^2).
<li><tt>CvData = 1</tt>: the flow coefficient is given by the metric Kv coefficient <tt>Kvnom</tt> (m^3/h).
<li><tt>CvData = 2</tt>: the flow coefficient is given by the US Cv coefficient <tt>Cvnom</tt> (USG/min).
<li><tt>CvData = 3</tt>: the flow coefficient is given implicitly by the operating point (<tt>wnom</tt>,<tt>dpnom</tt>, <tt>rhonom<tt/>).
</ul>
<p>The nominal pressure drop <tt>dpnom</tt> must always be specified; to avoid numerical singularities, the flow characteristic is modified for pressure drops less than <tt>b*dpnom</tt> (the default value is 1% of the nominal pressure drop). Increase this parameter if numerical instabilities occur in valves with very low pressure drops.
<p>If <tt>CheckValve</tt> is true, then the flow is stopped when the outlet pressure is higher than the inlet pressure; otherwise, reverse flow takes place.
<p>The default flow characteristic <tt>FlowChar</tt> is linear; this can be replaced by any user-defined function (e.g. equal percentage, quick opening, etc.).
</HTML>", revisions="<html>
<ul>
<li><i>1 Jul 2004</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Valve models restructured using inheritance. <br>
       Adapted to Modelica.Media.</li>
<li><i>1 Oct 2003</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       First release.</li>
</ul>
</html>"));
    // Default functions for valve characteristics
    function linear 
      input Real x;
      output Real y;
      annotation (derivative=der_linear);
    algorithm 
          y := x;
    end linear;
      
    function one 
      input Real x;
      output Real y;
        
      annotation (derivative=der_one);
    algorithm 
        y := 1;
    end one;
      
    function der_linear 
      input Real x;
      input Real der_x;
      output Real der_y;
    algorithm 
      der_y := der_x;
    end der_linear;
      
    function der_one 
      input Real x;
      input Real der_x;
      output Real der_y;
    algorithm 
      der_y := 0;
    end der_one;
      
    equation 
      if CvData == 0 then
        Av = Avnom;
      elseif CvData == 1 then
        Av = 2.7778e-5*Kvnom;
      elseif CvData == 2 then
        Av = 2.4027e-5*Cvnom;
      elseif CvData == 3 then
        Av = wnom/sqrt(rhonom*dpnom);
      end if;
      w = port_a.m_flow;
      // inlet.m_flow + outlet.m_flow = 0;
      // inlet.H_flow=semiLinear(inlet.m_flow,inlet.h,fluid.h);
      // outlet.H_flow=semiLinear(outlet.m_flow,outlet.h,fluid.h);
      // inlet.mXi_flow = semiLinear(inlet.m_flow, outlet.X_i, inlet.X_i);
      // outlet.mXi_flow + inlet.mXi_flow = zeros(Medium.nX_i);
      // inlet.H_flow+outlet.H_flow=0;
      // fluid.X_i = inlet.X_i;
      // fluid.p=inlet.p;
      Tin=medium_a.T;
      rho=medium_a.d;
    end ValveBase;

    model ValveLiquid "Valve for incompressible liquid flow" 
      extends ValveBase;
      Real z "Normalized pressure drop";
      Real sqrtz;
      annotation (
        Icon(Text(extent=[-100, -40; 100, -80], string="%name")),
        Diagram,
        Documentation(info="<HTML>
<p>Liquid water valve model according to the IEC 534/ISA S.75 standards for valve sizing, incompressible fluid. <p>
Extends the <tt>ValveBase</tt> model (see the corresponding documentation for common valve features).
</html>", revisions="<html>
<ul>
<li><i>27 Sep 2004</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted to Modelica_Fluid.</li>
<li><i>1 Jul 2004</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Valve model restructured using inheritance. <br>
       Adapted to Modelica.Media.</li>
<li><i>1 Oct 2003</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       First release.</li>
</ul>
</HTML>"));
    equation 
      z = (port_a.p - port_b.p)/dpnom;
      if CheckValve then
        sqrtz = (if z >= 0 then z/sqrt(z + b) else 0);
      else
        sqrtz = noEvent(z/sqrt(abs(z) + b));
      end if;
      w = FlowChar(theta)*Av*sqrt(rho*dpnom)*sqrtz;
    end ValveLiquid;
  protected 

    model ValveLiquidChoked 
      "Valve for liquid flow, allows choked flow conditions" 
      
      extends ValveBase(redeclare replaceable package Medium = 
                   Modelica.Media.Interfaces.PartialTwoPhaseMedium);
      parameter Real Flnom=0.9 "Liquid pressure recovery factor";
      replaceable function FlowChar = linear "Flow characteristic";
      replaceable function Flfun = one "Pressure recovery characteristic";
      SI.MassFlowRate w "Mass flowrate";
      Real Av;
      Real Ff;
      Real Fl;
      Real z "Normalized pressure drop";
      Real sqrtz;
      Medium.AbsolutePressure pv "Saturation pressure";
      Boolean chokedFlow "Choked flow conditions";
      
      annotation (
        Icon(Text(extent=[-100, -40; 100, -80], string="%name")),
        Diagram,
        Documentation(info="<HTML>
<p>Liquid water valve model according to the IEC 534/ISA S.75 standards for valve sizing, incompressible fluid, with possible choked flow conditions. <p>
Extends the <tt>ValveBase</tt> model (see the corresponding documentation for common valve features).<p>
The model operating range includes choked flow operation, which takes place for low outlet pressures due to flashing in the vena contracta; otherwise, non-choking conditions are assumed.
<p>The default liquid pressure recovery coefficient <tt>Fl</tt> is constant and given by the parameter <tt>Flnom</tt>. The relative change (per unit) of the recovery coefficient can be specified as a given function of the valve opening by customising the <tt>Flfun</tt> function.
</HTML>", revisions="<html>
<ul>
<li><i>27 Sep 2004</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted to Modelica_Fluid.</li>
<li><i>1 Jul 2004</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Valve model restructured using inheritance. <br>
       Adapted to Modelica.Media.</li>
<li><i>1 Oct 2003</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       First release.</li>
</ul>
</HTML>"));
    equation 
      if CheckValve then
        sqrtz = (if z >= 0 then z/sqrt(z + b) else 0);
      else
        sqrtz = noEvent(z/sqrt(abs(z) + b));
      end if;
      pv = Medium.saturationPressure(Tin);
      Ff = 0.96 - 0.28*sqrt(pv/22064.0e3);
      Fl = Flnom*Flfun(theta);
      chokedFlow= outlet.p < (1 - Fl^2)*inlet.p + Ff*Fl^2*pv;
      if chokedFlow then
        z = Fl^2*(inlet.p - Ff*pv)/dpnom;
      else
        z = (inlet.p - outlet.p)/dpnom;
      end if;
      w = FlowChar(theta)*Av*sqrt(rho*dpnom)*sqrtz;
      
    end ValveLiquidChoked;

    model ValveGas "Valve for steam flow" 
      extends ValveBase;
      parameter SI.Pressure pnom=0 "Nominal inlet pressure";
      parameter Real Fxtnom=0.5 "Nominal Fk*xt critical ratio";
      replaceable function xtfun = one "Critical ratio characteristic";
      Real Fxt;
      Real x;
      Real xs;
      Real Y;
      Real z "Normalized x";
      Real sqrtz;
      annotation (
        Icon(Text(extent=[-100, -40; 100, -80], string="%name")),
        Diagram,
        Documentation(info="<HTML>
<p>Liquid water valve model according to the IEC 534/ISA S.75 standards for valve sizing, compressible fluid. <p>
Extends the <tt>ValveBase</tt> model (see the corresponding documentation for common valve features).
<p>The product Fk*xt is given by the parameter <tt>Fxtnom</tt>, and is assumed constant by default. The relative change (per unit) of the xt coefficient with the valve opening can be specified by customising the <tt>xtfun</tt> function.
</HTML>", revisions="<html>
<ul>
<li><i>27 Sep 2004</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted to Modelica_Fluid.</li>
<li><i>1 Jul 2004</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Valve model restructured using inheritance. <br>
       Adapted to Modelica.Media.</li>
<li><i>1 Oct 2003</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       First release.</li>
</ul>
</HTML>"));
    equation 
      Fxt = Fxtnom*xtfun(theta);
      x = (inlet.p - outlet.p)/inlet.p;
      xs = if x < -Fxt then -Fxt else if x > Fxt then Fxt else x;
      Y = 1 - abs(xs)/(3*Fxt);
      z = xs/(dpnom/pnom);
      if CheckValve then
        sqrtz = (if z >= 0 then z/sqrt(z + b) else 0);
      else
        sqrtz = noEvent(z/sqrt(abs(z) + b));
      end if;
      w = FlowChar(theta)*Av*Y*sqrt(dpnom/pnom*inlet.p*rho)*sqrtz;
    end ValveGas;

    model ValveLin "Valve with linear pressure drop" 
     annotation (Icon(
          Line(points=[0, 40; 0, 0], style(
              color=0,
              thickness=2,
              fillPattern=1)),
          Polygon(points=[-80, 40; -80, -40; 0, 0; -80, 40], style(
              color=0,
              thickness=2,
              fillPattern=1)),
          Polygon(points=[80, 40; 0, 0; 80, -40; 80, 40], style(
              color=0,
              thickness=2,
              fillPattern=1)),
          Rectangle(extent=[-20, 60; 20, 40], style(
              color=0,
              fillColor=0,
              fillPattern=1))), Diagram,
        Documentation(revisions="<html>
<ul>
<li><i>27 Sep 2004</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted to Modelica_Fluid.</li>
<li><i>1 Oct 2003</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       First release.</li>
</ul>
</html>"));
      replaceable package Medium = Modelica.Media.Interfaces.PartialMedium 
        "Medium model" 
                   annotation(choicesAllMatching= true);
      parameter Real Kv(unit="kg/(s.Pa)") "Nominal hydraulic conductance";
      Modelica.SIunits.MassFlowRate w "Mass flowrate";
      Modelica.SIunits.SpecificEnthalpy h "Fluid specific enthalpy";
      Modelica_Fluid.Interfaces.FluidPort_a inlet(redeclare package Medium=Medium) 
                    annotation (extent=[-120, -20; -80, 20]);
      Modelica_Fluid.Interfaces.FluidPort_b outlet(redeclare package Medium=Medium) 
                     annotation (extent=[80, -20; 120, 20]);
      Modelica.Blocks.Interfaces.RealInput cmd 
        annotation (extent=[-20, 60; 20, 100], rotation=-90);
    equation 
      inlet.m_flow + outlet.m_flow = 0;
      w = inlet.m_flow;
      inlet.H_flow=semiLinear(inlet.m_flow,inlet.h,h);
      outlet.H_flow=semiLinear(outlet.m_flow,outlet.h,h);
      inlet.H_flow+outlet.H_flow=0;
      w = Kv*cmd*(inlet.p - outlet.p);
      annotation (
        Icon(Text(extent=[-100, -40; 100, -74], string="%name")),
        Diagram,
        Documentation(info="<HTML>
<p>This very simple model provides a pressure drop which is proportional to the flowrate and to the <tt>cmd</tt> signal, without computing any fluid property.</p>
</HTML>", revisions="<html>
<ul>
<li><i>1 Oct 2003</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       First release.</li>
</ul>
</html>"));
    end ValveLin;
  public 

    model SpecialTank "Tank with one bottom inlet/outlet" 
      import Modelica.SIunits.Conversions.*;
      
      replaceable package Medium = Modelica_Fluid.PackageMedium extends 
        Modelica.Media.Interfaces.PartialMedium "Medium in the component" 
        annotation (choicesAllMatching=true);
      
      Modelica_Fluid.Interfaces.FluidPort_b port(redeclare package Medium = Medium) 
        annotation (extent=[-10, -120; 10, -100], rotation=90);
      Medium.BaseProperties medium(
        preferredMediumStates=true,
        p(start=p_ambient),
        T(start=T_start),
        Xi(start=X_start[1:Medium.nXi]));
      parameter Boolean initLevel = true 
        "true if level is initialized in this tank";
      parameter Modelica.SIunits.Area area "Tank area";
      parameter Medium.AbsolutePressure p_ambient=101325 
        "Tank surface pressure";
      parameter Modelica.SIunits.Height level_start(min=0) "Initial tank level"
        annotation(Dialog(group=Initialization));
      parameter Medium.Temperature T_start=from_degC(20) 
        "Initial tank temperature" 
        annotation(Dialog(group=Initialization));
      parameter Medium.MassFraction X_start[Medium.nX](quantity=Medium.
            substanceNames) = zeros(Medium.nX) 
        "Initial independent tank mass fractions m_i/m" 
        annotation(Dialog(group=Initialization),enable=(Medium.nXi>0));
      constant Modelica.SIunits.Acceleration g=Modelica.Constants.g_n;
      Modelica.SIunits.Height level(stateSelect=StateSelect.prefer, min=0) 
        "Level height of tank";
      Modelica.SIunits.Energy U "Internal energy of tank volume";
      Modelica.SIunits.Volume V(stateSelect=StateSelect.never) 
        "Actual tank volume";
      Real m(quantity=Medium.mediumName, unit="kg") "Mass of tank volume";
      Real mX[Medium.nX](quantity=Medium.substanceNames, each unit="kg") 
        "Component masses of the independent substances";
    initial equation 
      if not Medium.singleState then
        mX = m*X_start[1:Medium.nXi];
      end if;
      if initLevel then
         level = level_start;
      end if;
      medium.T = T_start;
      medium.Xi = X_start[1:Medium.nXi];
    equation 
      port.p = medium.p;
      
      /* Handle reverse and zero flow */
      port.H_flow = semiLinear(port.m_flow, port.h, medium.h);
      port.mXi_flow = semiLinear(port.m_flow, port.Xi, medium.X);
      
      /*
  More precise equations (test later):
  Momentum balance
  (integrated momentum equation for frictionless fluid with density that is
   independent of the level, i.e., the unsteady Bernoulli equation for incompressible fluid)
  v_level = der(level);
  v = -port.m_flow/(rho*A_outlet);
  level*der(v_level) + (v^2 - v_level^2)/2 - g*level + (p - p_ambient)/rho = 0;
  Energy balance
  Potential energy: E_pot = integ(dm*g*s)
                          = g*integ(rho*A*s*ds)
                          = g*rho*A*z^2/2
  Kinetic energy  : E_kin = integ(dm*v^2/2)
                          = integ(rho*A*v^2/2*ds)
                          = rho*A*v^2/2*integ(ds)
                          = rho*A*v^2/2*z
                          = M*v^2/2
  E = U + M*g*z/2 + M*v_level^2/2
  der(E) = port.H_flow + port.m_flow*v^2/2 - p_ambient*area*der(level)
*/
      
      V = area*level;
      m = V*medium.d;
      mX = m*medium.Xi;
      U = m*medium.u;
      
      // Mass balance
      der(m) = port.m_flow;
      der(mX) = port.mXi_flow;
      
      // Momentum balance
      medium.p = m*g/area + p_ambient;
      
      // Energy balance
      der(U) = port.H_flow - 0.5*(p_ambient+medium.p)*der(V);
      
      annotation (
        Icon(
          Rectangle(extent=[-100, 90; 100, 26], style(color=7, fillColor=7)),
          Rectangle(extent=[-100, 26; 100, -100], style(
              color=69,
              fillColor=69,
              fillPattern=1)),
          Line(points=[-100, 100; -100, -100; 100, -100; 100, 100], style(
              color=0,
              fillColor=69,
              fillPattern=1)),
          Text(
            extent=[-112, 162; 122, 102],
            string="%name",
            style(fillColor=69, fillPattern=1)),
          Text(
            extent=[-86, -38; 94, -78],
            style(color=0),
            string="%level_start"),
          Text(
            extent=[-94, 78; 94, 38],
            style(color=0),
            string="%p_ambient"),
          Text(
            extent=[-94, 14; 90, -2],
            string="level_start =",
            style(color=0))),
        Documentation(info="<HTML>
<p>
This is a simplified model of a tank. The top part is open to the environment.
The tank is filled with a single or multiple-substance liquid.
The whole tank is assumed to have uniform temperature and mass fractions.
</p>
</HTML>"),
        Diagram);
      
    end SpecialTank;
  end Components;
  annotation (uses(Modelica(version="2.2"), Modelica_Fluid(version="0.900")));
  extends Modelica.Icons.Library;
  model Exercise1 "model for exercise 1" 
    annotation (
      Icon(Text(
          extent=[-84,82; 86,-84],
          string="E1",
          style(color=0, rgbcolor={0,0,0}))),
      Diagram,
      experiment(StopTime=4),
      experimentSetupOutput,
      Documentation(info="<html>
<body>
<h3>Structural and numerical properties of different fluid models</h3>
<br>
This very simple model is used to demonstrate a number of structural
and numerical properties of&nbsp; combining different fluid media
models with the same component models. In order to observe the
proeprties in detail, some flags have to be set (this is done also when
you run setup.mos)<br>
<ol>
<li>Set the flag <b>LogStateSelection</b>:=true
(Translation tab in simulation setup)</li>
<li>Most media models also need <b>Hidden.AutomaticDifferentiation</b> := true;</li>
</ol>
Perform the following tasks with the model:<br>
<ol>
<li>Run the model with the default medium (high accuracy water with
pressure and temperature as states). Check that the states are really p
and T. Note also that these are <span style=\"font-weight: bold;\">2
states per volume</span>.</li>
<li>Run the model with the medium \"1,2-Propylene glycol, 47% mixture
with water\". Select all components except the signal sources by
dragging across and select from the drop-down menu. Check what states
are obtained. Compare the temperature trajectories.</li>
<li>Run the model&nbsp; with the medium \"simple natural gas mix with
6 components\", selected froim the drop-down menu. Check what states are
obtained.</li>
<li><span style=\"font-weight: bold;\">Optional exercise:</span> Go to
Modelica.Media.TableBased.Examples. Duplicate the Class Glykol47, and
add the following modifier to it: \"enthalpyOfT=false,\"<br>
The overall should look like:<br>
<pre>package Glycol47 \"modified glycol, 47% mixture with water\" <br>&nbsp; extends Modelica.Media.Incompressible.TableBased(<br>&nbsp;&nbsp;&nbsp; mediumName=\"Glycol-Water 47%\",<br>&nbsp;&nbsp;&nbsp; enthalpyOfT=false,<br>&nbsp;&nbsp;&nbsp; T_min=Modelica.SIunits.Conversions.from_degC(-30),<br>&nbsp;&nbsp;&nbsp; T_max=Modelica.SIunits.Conversions.from_degC(100),<br>&nbsp;&nbsp;&nbsp; TinK=false,<br>&nbsp;&nbsp;&nbsp; T0=273.15<br>&nbsp;&nbsp;&nbsp; tableDensity=[-30,1066; -20,1062; -10,1058; 0,1054; 20,1044; 40,1030; 60,<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; 1015; 80,999; 100,984],</pre>
<span style=\"font-family: monospace;\"></span>...<br>
<pre>end Glycol47;<br></pre>
Note the modified description string that shows up in the drop down
menu for medium selection. This change has the consequence that the
pressure dependence of the enthalpy is included in the model. Compare
the temperature trajectory, and the simulation time. For allmost all
incompressible media it is clear that the pressure dependence is
negligible.<br>
</li>
</ol>
</body>
</html>"));
    Components.ValveLiquid V1(
      dpnom=5e5,
      rhonom=1000,
      CvData=3,
      wnom=1.2,
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                annotation (extent=[-52,0; -32,20]);
    Modelica_Fluid.Interfaces.PortVolume volume(
      h_start=1e5,
      medium(h(start=2.0e5)),
      T_start=300,
      p_start=7e5,
      V=1e-4,
      use_T_start=true,
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[-14,0; 6,20]);
    Components.ValveLiquid V2(
      dpnom=4e5,
      wnom=2,
      rhonom=1000,
      CvData=3,
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                annotation (extent=[26,0; 46,20]);
    Modelica.Blocks.Sources.Ramp Close_V1(
      duration=2,
      offset=1,
      startTime=1,
      height=-0.99) 
                  annotation (extent=[-74,40; -54,60]);
    Modelica.Blocks.Sources.Ramp Close_V2(
      duration=1,
      height=-0.99,
      offset=1,
      startTime=1)    annotation (extent=[6,40; 26,60]);
    Modelica_Fluid.Sources.FixedAmbient Source_1(
      p_ambient=10.0e5,
      use_T_ambient=true,
      h_ambient=1.0e5,
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[-100,0; -80,20]);
    Modelica_Fluid.Sources.FixedAmbient Sink(
      use_T_ambient=true,
      h_ambient=1.0e5,
      p_ambient=1.0e5,
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[100,0; 80,20]);
  equation 
    connect(Close_V2.y, V2.theta) annotation (points=[27,50; 36,50; 36,18],
        style(color=74, rgbcolor={0,0,127}));
    connect(Close_V1.y, V1.theta) annotation (points=[-53,50; -42,50; -42,18],
        style(color=74, rgbcolor={0,0,127}));
    connect(volume.port, V2.port_a) annotation (points=[-4,10; 25,10], style(
          color=69, rgbcolor={0,127,255}));
    connect(V1.port_b, volume.port) annotation (points=[-31,10; -4,10], style(
          color=69, rgbcolor={0,127,255}));
    connect(Source_1.port, V1.port_a) annotation (points=[-79,10; -53,10],
        style(color=69, rgbcolor={0,127,255}));
    connect(V2.port_b, Sink.port) annotation (points=[47,10; 79,10], style(
          color=69, rgbcolor={0,127,255}));
  end Exercise1;
  
  model Exercise2 "model for exercise 2" 
    annotation (Icon(
           Text(
          extent=[-84,82; 86,-84],
          style(color=0, rgbcolor={0,0,0}),
          string="E2")), Diagram,
      experiment(StopTime=4, Tolerance=1e-006),
      experimentSetupOutput,
      Documentation(info="<html>
<body>
<h3>Numerical Issues of Reversing Flow</h3>
Everybody knows text book equations. Not everybody knows, that almost
all text book equations should have a big red disclaimer tagged to it:
will not work as advertised unless subtly modified.&nbsp; Here is the
prototypical form of this problem, the root function singularity. For
convenience, the \"bad\" form of the model is already supplied. Perform
the following exercises:<br>
<ol>
<li>Run the model as is. It will simulate until time 1.999 and then
aparently stop. Open the small simulation window and hit Ctrl-C for a
couple of times. Simulation time essentially stalls.</li>
<li>Check that the model runs fine for only positive and only
negative mass flow (Set variable flowsource.height and
flowsource.offset to achieve this).</li>
<li>Implement a regularized model using if-statements (you need 3
branches). For convenience, we provide the expression for regularizing
around 0 pressure drop:<br>
m_flow = (C1*dp + C3*dp^3)/sqrt(k); <br>
C3 = -0.25/(sqrt(dp_small)*dp_small^2);<br>
C1 = 0.5/sqrt(dp_small) - 3.0*C3*dp_small^2;</li>
<li>Do numerical experiments to show that a very small value for the
parameter dp_small does not work either.<br>
<br>
</li>
</ol>
</body>
</html>"));
    Modelica_Fluid.Sources.FixedAmbient Sink(
      use_p_ambient=true,
      redeclare package Medium = Modelica.Media.Water.WaterIF97_pT,
      p_ambient=5.0e5) 
      annotation (extent=[60,0; 40,20]);
    Modelica_Fluid.Sources.PrescribedMassFlowRate_TX MassFlowSource(redeclare 
        package Medium = Modelica.Media.Water.WaterIF97_pT) 
      annotation (extent=[-80,0; -60,20]);
    Modelica.Blocks.Sources.Ramp flowsource(
      duration=2,
      startTime=1,
      offset=1,
      height=-2)  annotation (extent=[-100,40; -80,60]);
    Components.BadOrifice orifice(redeclare package Medium = 
          Modelica.Media.Water.WaterIF97_pT) annotation (extent=[0,0; 20,20]);
    Components.HeatTransferVolume volume(redeclare package Medium = 
          Modelica.Media.Water.WaterIF97_pT, p_start=6.0e5) 
                                             annotation (extent=[-40,0; -20,20]);
  equation 
    connect(flowsource.y, MassFlowSource.m_flow_ambient) 
                                                   annotation (points=[-79,50;
          -68,50; -68,30; -94,30; -94,10; -82,10], style(
        color=74,
        rgbcolor={0,0,127},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(orifice.port_b, Sink.port) annotation (points=[21,10; 39,10], style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(orifice.port_a, volume.flowport) annotation (points=[-1,10; -30,10],
        style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(volume.flowport, MassFlowSource.port) annotation (points=[-30,10; -59,
          10], style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
  end Exercise2;
  
  model Exercise3_1 "model for exercise 3" 
    annotation (Icon(
           Text(
          extent=[-96,94; 100,-82],
          style(color=0, rgbcolor={0,0,0}),
          string="E3")), Diagram,
      experiment(StopTime=4, Tolerance=1e-006),
      experimentSetupOutput,
      Documentation(info="<html>
<body>
<h3>Index reduction in thermodynamic models</h3>
<p>Index reduction (seemingly independent dynamic states in the
equations
are coupled via algebraic constraints) are typical in in mechanical
contexts,
but not usually in thermodynamics. Nonetheless, it can be very
convenient if a
tool can handle such situations automatically. In particular for low
order,
control-oriented models this is desirable.
</p>
<p>However, in some situations and depending the complexity of the
fluid property computation,
it can be better to do the model simplification \"by hand\". This can be
as easy as adding two volumes,
but in some situations the automatic version is the only reasonable way
to do it</p>
<p>There are two recommended examples related to index reduction:<br>
</p>
<ol>
<li>A wall and a volume have coupled temperatures, i.e. the
assumption is excellent (infinite) heat transfer</li>
<li>The second example is two volumes in series. This can be
unavoidable when coupling complex models. <br>
</li>
<li>The third example is given by tanks connected at the bottom. Here, the situation
is different: the pressures at the botttom are the same, but not the temperatures.
The reason that this situation is handled differently is the use of semiLinear(m_flow,h1,h2)
for flow variables at the tank connector.</li>
</ol>
<h4>Exercises</h4>
Run the model as it is given in Exercise 3_1 in the package ThermodynamicTutorial. 
Note that the wall is connected without any heat resistance to the volume.
Thus, the temperatures are the same and therefore we have one less independent state. Check the variable <p>LogStateSelection</b> := true; Record what states are chosen in all of the following cases, and observe the dynamics in the volume.
<ol>
<li>Remove the connection between the volume and wall model and re-run</li>
<li>Insert the element Modelica.Thermal.HeatTransfer.Convection between volume and wall, add a constant
for the heat transfer coefficient and re-run</li>
<li>Change to different fluids with different independent variables, re-run with each of them. <b>Attention</b> </li>
<ul>
<li>Make sure to initialize with the proper variable (flag <b>use_T_start</b>)</li>
<li>Make sure to have appropriate,legal values for T or h</li>
</ul>
</ol>
<h4>Initialization</h4>
<p>Whenever an index reduction occurs, the number of initial conditions is
reduced accordingly. In this case, only one of the components wall or volume can 
have an active initial condition for the temperature T

</body>
</html>"));
    Modelica_Fluid.Sources.FixedAmbient Sink(
      use_p_ambient=true,
      h_ambient=1.5e6,
      use_T_ambient=true,
      p_ambient=5.5e5,
      redeclare package Medium = Modelica.Media.Water.StandardWater) 
      annotation (extent=[100,-20; 80,0]);
    
    Modelica.Blocks.Sources.Ramp flowsource(
      duration=2,
      startTime=1,
      height=-0.2,
      offset=0.4) annotation (extent=[-100,20; -80,40]);
    Components.HeatTransferVolume volume(
      p_start=7.0e5,
      V=1e-5,
      h_start=1.5e6,
      use_p_start=true,
      redeclare package Medium = Modelica.Media.Water.StandardWater,
      use_T_start=false)                     annotation (extent=[-30,-20; -10,0]);
    
    Modelica_Fluid.Components.ShortPipe pipe(
      dp_nominal=5000,
      length=2.5,
      dp(start=500),
      roughness=0.000045,
      diameter=0.02,
      from_dp=true,
      p_small=10,
      frictionType=Modelica_Fluid.Types.FrictionTypes.ConstantTurbulent,
      redeclare package Medium = Modelica.Media.Water.StandardWater) 
      annotation (extent=[28,-20; 48,0]);
    
    Modelica.Thermal.HeatTransfer.HeatCapacitor wall(
      C=5000.0,
      T(start=300.0),
      steadyStateStart=false,
      port(T(start=300.0))) 
                annotation (extent=[-40,36; 0,76]);
    Modelica_Fluid.Sources.PrescribedMassFlowRate_TX PrescribedMassFlowRate_TX1
      (redeclare package Medium = Modelica.Media.Water.StandardWater) 
      annotation (extent=[-80,-20; -60,0]);
  equation 
    connect(pipe.port_b, Sink.port) annotation (points=[49,-10; 79,-10],
                                                                       style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(volume.flowport, pipe.port_a) annotation (points=[-20,-10; 27,-10],
        style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(PrescribedMassFlowRate_TX1.port, volume.flowport) annotation (
        points=[-59,-10; -20,-10], style(color=69, rgbcolor={0,127,255}));
    connect(flowsource.y, PrescribedMassFlowRate_TX1.m_flow_ambient) 
      annotation (points=[-79,30; -70,30; -70,8; -96,8; -96,-10; -82,-10],
        style(color=74, rgbcolor={0,0,127}));
    connect(wall.port, volume.heatport) annotation (points=[-20,36; -20,-2], 
        style(
        color=42, 
        rgbcolor={191,0,0}, 
        thickness=2));
  end Exercise3_1;
  
  model Exercise3_2 "model for exercise 3" 
    annotation (Icon(
           Text(
          extent=[-84,82; 86,-84],
          style(color=0, rgbcolor={0,0,0}),
          string="E3")), Diagram(Text(
          extent=[-88,-30; 88,-84],
          string=
              "Index reduction with perfect mixing of the energy and identical pressure",
          style(color=0, rgbcolor={0,0,0})), Text(
          extent=[-88,-44; 42,-92],
          style(color=0, rgbcolor={0,0,0}),
          string="4 states from 2 volumes are reduced to 2 states")),
      experiment(StopTime=4, Tolerance=1e-006),
      experimentSetupOutput,
      Documentation(info="<html>
<body>
<h3>Index reduction in thermodynamic models</h3>
<p>Index reduction (seemingly independent dynamic states in the
equations
are coupled via algebraic constraints) are typical in in mechanical
contexts,
but not usually in thermodynamics. Nonetheless, it can be very
convenient if a
tool can handle such situations automatically. In particular for low
order,
control-oriented models this is desirable.
</p>
<p>However, in some situations and depending the complexity of the
fluid property computation,
it can be better to do the model simplification \"by hand\". This can be
as easy as adding two volumes,
but in some situations the automatic version is the only reasonable way
to do it</p>
<p>There are two recommended examples related to index reduction:<br>
</p>
<ol>
<li>A wall and a volume have coupled temperatures, i.e. the
assumption is excellent (infinite) heat transfer</li>
<li>The second example is two volumes in series. This can be
unavoidable when coupling complex models. <br>
</li>
<li>The third example is given by tanks connected at the bottom. Here, the situation
is different: the pressures at the botttom are the same, but not the temperatures.
The reason that this situation is handled differently is the use of semiLinear(m_flow,h1,h2)
for flow variables at the tank connector.</li>
</ol>
<h4>Exercises</h4>
Run the model as it is given in Exercise 3_2 in the package ThermodynamicTutorial. 
Note that the volumes are connected without any pressure drop or heat resistance.
Thus, the enthalpies and pressures are the same and therefore we have two less independent states. Check the variable <p>LogStateSelection</b> := true; Record what states are chosen, and observe the dynamics in the volume.
<ol>
<li>Change the size of the volumes and re-run</li>
<li>Insert the element Modelica_Fluid.Components.ShortPipe between volume and wall and re-run</li>
<li>Change to different fluids with different independent variables, re-run with each of them. <b>Attention</b> </li>
<ul>
<li>Make sure to initialize with the proper variable (flag <b>use_T_start</b>)</li>
<li>Make sure to have appropriate,legal values for T or h</li>
</ul>
</ol>
h4>Initialization</h4>
Whenever an index reduction occurs, the number of initial conditions is
reduced accordingly. This is very clear from the example were two
volumes are combined into one. Onle one pressure and one temperature
can be set fixed, not independent ones.<br>

</body>
</html>"));
    
    Modelica_Fluid.Sources.FixedAmbient Sink(
      use_p_ambient=true,
      p_ambient=5.0e5,
      redeclare package Medium = Modelica.Media.Water.StandardWater) 
      annotation (extent=[100,0; 80,20]);
    Modelica_Fluid.Sources.PrescribedMassFlowRate_TX MassFlowSource(redeclare 
        package Medium = Modelica.Media.Water.StandardWater) 
      annotation (extent=[-80,0; -60,20]);
    Modelica.Blocks.Sources.Ramp flowsource(
      duration=2,
      startTime=1,
      offset=1,
      height=-0.8) 
                  annotation (extent=[-100,40; -80,60]);
    Components.HeatTransferVolume volume(    p_start=6.0e5,
      V=1e-5,
      initializeVolume=true,
      redeclare package Medium = Modelica.Media.Water.StandardWater) 
                                             annotation (extent=[-30,0; -10,20]);
    Modelica_Fluid.Components.ShortPipe pipe(redeclare package Medium = 
          Modelica.Media.Water.StandardWater) 
      annotation (extent=[26,0; 46,20]);
    Components.HeatTransferVolume volume2(
      V=1e-5,
      use_p_start=true,
      use_T_start=true,
      redeclare package Medium = Modelica.Media.Water.StandardWater,
      initializeVolume=false) 
      annotation (extent=[-30,-24; -10,-44]);
  equation 
    connect(flowsource.y, MassFlowSource.m_flow_ambient) 
                                                   annotation (points=[-79,50;
          -68,50; -68,30; -94,30; -94,10; -82,10], style(
        color=74,
        rgbcolor={0,0,127},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(volume.flowport, MassFlowSource.port) annotation (points=[-20,10;
          -59,10],
               style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(pipe.port_b, Sink.port) annotation (points=[47,10; 79,10], style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(volume.flowport, pipe.port_a) annotation (points=[-20,10; 25,10],
        style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(volume2.flowport, volume.flowport) annotation (points=[-20,-34; -20,
          10], style(color=69, rgbcolor={0,127,255}));
  end Exercise3_2;
  
  model Exercise3_3 "model for exercise 3" 
    annotation (Icon(
           Text(
          extent=[-84,82; 86,-84],
          style(color=0, rgbcolor={0,0,0}),
          string="E3")), Diagram,
      experiment(StopTime=500, Tolerance=1e-006),
      experimentSetupOutput,
      Documentation(info="<html>
<body>
<h3>Index reduction in thermodynamic models</h3>
<p>Index reduction (seemingly independent dynamic states in the
equations
are coupled via algebraic constraints) are typical in in mechanical
contexts,
but not usually in thermodynamics. Nonetheless, it can be very
convenient if a
tool can handle such situations automatically. In particular for low
order,
control-oriented models this is desirable.
</p>
<p>However, in some situations and depending the complexity of the
fluid property computation,
it can be better to do the model simplification \"by hand\". This can be
as easy as adding two volumes,
but in some situations the automatic version is the only reasonable way
to do it</p>
<p>There are two recommended examples related to index reduction:<br>
</p>
<ol>
<li>A wall and a volume have coupled temperatures, i.e. the
assumption is excellent (infinite) heat transfer</li>
<li>The second example is two volumes in series. This can be
unavoidable when coupling complex models. <br>
</li>
<li>The third example is given by tanks connected at the bottom. Here, the situation
is different: the pressures at the botttom are the same, but not the temperatures.
The reason that this situation is handled differently is the use of semiLinear(m_flow,h1,h2)
for flow variables at the tank connector.</li>
</ol>
<h4>Exercises</h4>
Run the model as it is given in Exercise 3_3 in the package ThermodynamicTutorial, for 500 seconds
(settings are stored). For all runs, observe what states are selected and what equation systems occur.
<ol>
<li>Run the model with a compressible medium from WaterIF97. This is not a particularly useful
approximation, but shows that the models are written in a general way.</li>
<li>Remove the pipe labeled pipeRemove and connect the tanks. What is the approximation made in this case? <b>Attention</b> </li>
<ul>
<li>Make sure to remove the redundant initial condition (flag <b>initLevel</b> = false) in any one of the tanks.</li>
</ul>
<li>Change to different fluids with different independent variables, re-run with each of them.
</li></ol>
<h4>Initialization</h4>
Whenever an index reduction occurs, the number of initial conditions is
reduced accordingly. This example is somewhat artificial, so it uses a special component from this course. In this case the initialization for one of the tank levels has to be removed by setting <b>initLevel</b> to false<br>
</body>
</html>"));
    Modelica_Fluid.Sources.FixedAmbient Sink(
      use_p_ambient=true,
      p_ambient=1.2e5, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[100,0; 80,20]);
    
    Modelica_Fluid.Sources.PrescribedMassFlowRate_TX MassFlowSource(redeclare 
        package Medium = Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[-68,0; -48,20]);
    Modelica.Blocks.Sources.Ramp flowsource(
      duration=2,
      startTime=1,
      offset=1,
      height=-0.8) 
                  annotation (extent=[-100,0; -80,20]);
    Modelica_Fluid.Components.ShortPipe pipe(
      dp_nominal=10000,
      m_flow_nominal=10, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[44,0; 64,20]);
    
    Components.SpecialTank SpecialTank1(
      level_start=2,
      area=1, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47, 
      initLevel=false) 
                      annotation (extent=[16,36; 36,56]);
    
    Components.SpecialTank SpecialTank2(
      area=1,
      level_start=3, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[-34,46; -14,66]);
    
    Modelica_Fluid.Components.ShortPipe pipeRemove(
      dp_nominal=10000, 
      m_flow_nominal=10, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[-12,0; 8,20]);
  equation 
    connect(flowsource.y, MassFlowSource.m_flow_ambient) 
                                                   annotation (points=[-79,10; 
          -70,10],                                 style(
        color=74,
        rgbcolor={0,0,127},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(pipe.port_b, Sink.port) annotation (points=[65,10; 79,10], style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(pipe.port_a, SpecialTank1.port) annotation (points=[43,10; 26,10; 
          26,35], style(
        color=69, 
        rgbcolor={0,127,255}, 
        thickness=2));
    connect(MassFlowSource.port, SpecialTank2.port) annotation (points=[-47,10; 
          -24,10; -24,45], style(
        color=69, 
        rgbcolor={0,127,255}, 
        thickness=2));
    connect(pipeRemove.port_b, SpecialTank1.port) annotation (points=[9,10; 26,
          10; 26,35], style(
        color=69, 
        rgbcolor={0,127,255}, 
        thickness=2));
    connect(pipeRemove.port_a, SpecialTank2.port) annotation (points=[-13,10; 
          -24,10; -24,45], style(
        color=69, 
        rgbcolor={0,127,255}, 
        thickness=2));
  end Exercise3_3;
  
  model Exercise4 "model for exercise 4" 
    annotation (Icon(
           Text(
          extent=[-84,82; 86,-84],
          style(color=0, rgbcolor={0,0,0}),
          string="E4")), Diagram,
      experiment(StopTime=4, Tolerance=1e-006),
      experimentSetupOutput,
      Documentation(info="<html>
<body>
<h3>Non-linear systems of equations</h3>
<p>In this model a few pressure drop models are combined with one junction point and connected directly.
as can be seen from the log of the equation systems, </p>

<h4>Exercises</h4>
Run the model as it is given in Exercise 4. Repeat with other liquids, e.g. water with states p and h or p and T and incompressible media like Glycol. What are the sizes of the non-linear equation systems? Why do they differ?
<p><b>Revision history:</b></p>
<ul>
<li><i>March 3 2005</i>
    by <a href=\"mailto:hubertus@modelon.se\">Hubertus Tummescheit</a>:<br>
</body>
</html>"));
    Modelica_Fluid.Sources.FixedAmbient Sink(
      use_p_ambient=true,
      p_ambient=5.0e5, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[100,0; 80,20]);
    Modelica_Fluid.Sources.PrescribedMassFlowRate_TX MassFlowSource(redeclare 
        package Medium = Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[-80,0; -60,20]);
    Modelica.Blocks.Sources.Ramp flowsource(
      duration=2,
      startTime=1,
      offset=1,
      height=0)   annotation (extent=[-100,40; -80,60]);
    Components.HeatTransferVolume volume(    p_start=6.0e5,
      V=1e-5, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                                             annotation (extent=[-46,0; -26,20]);
    Modelica_Fluid.Components.ShortPipe pipe(
      roughness=0.0002,
      diameter=0.03, 
      frictionType=Modelica_Fluid.Types.FrictionTypes.ConstantTurbulent, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                     annotation (extent=[0,0; 20,20]);
    Modelica_Fluid.Components.ShortPipe pipe1(
      roughness=0.0002,
      diameter=0.03, 
      frictionType=Modelica_Fluid.Types.FrictionTypes.ConstantTurbulent, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                     annotation (extent=[40,0; 60,20]);
    Modelica_Fluid.Components.ShortPipe pipe2(
      roughness=0.0002,
      diameter=0.03, 
      frictionType=Modelica_Fluid.Types.FrictionTypes.ConstantTurbulent, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                     annotation (extent=[38,-40; 58,-20]);
    Modelica_Fluid.Sources.FixedAmbient Sink1(
      use_p_ambient=true, 
      p_ambient=4.5e5, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[100,-40; 80,-20]);
  equation 
    connect(flowsource.y, MassFlowSource.m_flow_ambient) 
                                                   annotation (points=[-79,50;
          -68,50; -68,30; -94,30; -94,10; -82,10], style(
        color=74,
        rgbcolor={0,0,127},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(volume.flowport, MassFlowSource.port) annotation (points=[-36,10;
          -59,10],
               style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(volume.flowport, pipe.port_a) annotation (points=[-36,10; -1,10],
        style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(pipe.port_b, pipe1.port_a) annotation (points=[21,10; 39,10], style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(pipe1.port_b, Sink.port) annotation (points=[61,10; 79,10], style(
        color=69,
        rgbcolor={0,127,255},
        thickness=2,
        gradient=2,
        fillColor=71,
        rgbfillColor={85,170,255}));
    connect(pipe2.port_b, Sink1.port) annotation (points=[59,-30; 79,-30], 
        style(
        color=69, 
        rgbcolor={0,127,255}, 
        thickness=2));
    connect(pipe2.port_a, pipe.port_b) annotation (points=[37,-30; 30,-30; 30,
          10; 21,10], style(
        color=69, 
        rgbcolor={0,127,255}, 
        thickness=2));
  end Exercise4;
  
protected 
package Glycol47 "modified glycol, 47% mixture with water" 
  extends Modelica.Media.Incompressible.TableBased(
    mediumName="Glycol-Water 47%",
    enthalpyOfT=false,
    T_min=Modelica.SIunits.Conversions.from_degC(-30),
    T_max=Modelica.SIunits.Conversions.from_degC(100),
    TinK=false,
    T0=273.15,
    tableDensity=[-30,1066; -20,1062; -10,1058; 0,1054; 20,1044; 40,1030; 60,
        1015; 80,999; 100,984],
    tableHeatCapacity=[-30,3450; -20,3490; -10,3520; 0,3560; 20,3620; 40,3690;
        60,3760; 80,3820; 100,3890],
    tableConductivity=[-30,0.397; -20,0.396; -10,0.395; 0,0.395; 20,0.394; 40,
        0.393; 60,0.392; 80,0.391; 100,0.390],
    tableViscosity=[-30,0.160; -20,0.0743; -10,0.0317; 0,0.0190; 20,0.00626; 40,
        0.00299; 60,0.00162; 80,0.00110; 100,0.00081],
    tableVaporPressure=[0,500; 20,1.9e3; 40,5.3e3; 60,16e3; 80,37e3; 100,80e3]);
end Glycol47;
  
public 
  model ValveReversingFlows "Test case for valves" 
    package Medium = Modelica.Media.Water.StandardWater;
    Modelica_Fluid.Sources.FixedAmbient_phX SourceP1(
      p_ambient=10e5,
      h_ambient=1e5, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[-100,30; -82,52]);
    Modelica_Fluid.Sources.FixedAmbient_phX SourceP2(
      p_ambient=8e5,
      h_ambient=1e5, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[-100,-50; -84,-30]);
    Modelica_Fluid.Sources.FixedAmbient_phX SinkP1(
      p_ambient=1e5,
      h_ambient=1e5, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[76,0; 60,20]);
    Components.ValveLiquid V1(
      rhonom=1000,
      CvData=3,
      dpnom=9e5,
      wnom=1.5, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                annotation (extent=[-50, 58; -30, 78]);
    Components.ValveLiquid V2(
      dpnom=5e5,
      rhonom=1000,
      CvData=3,
      wnom=1.2, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                annotation (extent=[-38, 26; -18, 46]);
    Components.ValveLiquid V3(
      dpnom=3e5,
      rhonom=1000,
      CvData=3,
      wnom=1.1, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                annotation (extent=[-38, -38; -18, -18]);
    Components.ValveLiquid V4(
      dpnom=8e5,
      rhonom=1000,
      CvData=3,
      wnom=1.3, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                annotation (extent=[-38, -78; -18, -58]);
    Components.ValveLiquid V5(
      dpnom=4e5,
      wnom=2,
      rhonom=1000,
      CvData=3, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
                annotation (extent=[28,0; 48,20]);
    annotation (
      Diagram,
      experiment(StopTime=4, Tolerance=1e-006),
      Documentation(info="<html>
<body>
<h3>Flow reversal for a liquid valve</h3>
<p>This model tests the <tt>ValveLiq</tt> model zero or reverse flow conditions.
<p>Simulate the model for 4 s. At t = 1 s the V5 valve closes in 1 s, the V2 and V3 valves close in 2 s and the V1 and V4 valves open in 2 s. The flow in valve V3 reverses between t = 1.83 and t = 1.93.
<h4>Exercises</h4>
Run the model as it is given in Exercise 5. Repeat with other liquids, e.g. water with states p and h.
Why is there one large non-linear system of equations (size 40 for incompressible table based media)? Can you avoid it by modifying the model?

<br>
<p><b>Revision history:</b></p>
<ul>
<li><i>1 Oct 2003</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco
Casella</a>:<br>
       First release.</li>
</ul>
<body>
</HTML>"), 
      Icon(Text(
          extent=[-86,86; 84,-80],
          style(color=0, rgbcolor={0,0,0}), 
          string="E5")));
    Modelica_Fluid.Sources.FixedAmbient_phX SinkP2(
      p_ambient=1e5,
      h_ambient=1e5, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[2,58; -16,78]);
    Modelica_Fluid.Sources.FixedAmbient_phX SinkP3(
      p_ambient=1e5,
      h_ambient=1e5, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[14,-78; -2,-58]);
    Modelica.Blocks.Sources.Ramp CloseLoad(
      duration=1,
      height=-0.99,
      offset=1,
      startTime=1)    annotation (extent=[8, 28; 28, 48]);
    Modelica.Blocks.Sources.Ramp OpenRelief(
      duration=2,
      height=1,
      offset=0,
      startTime=1) 
                  annotation (extent=[-94,74; -74,94]);
    Modelica.Blocks.Sources.Ramp CloseValves(
      duration=2,
      height=-1,
      offset=1,
      startTime=1) 
                  annotation (extent=[-96, -12; -76, 8]);
    Modelica_Fluid.Interfaces.PortVolume volume(
      h_start=1e5,
      medium(h(start=2.0e5)),
      T_start=300,
      use_T_start=false,
      p_start=7e5,
      V=1e-4, 
      redeclare package Medium = 
          Modelica.Media.Incompressible.Examples.Glycol47) 
      annotation (extent=[-12,-2; 8,20]);
  equation 
    connect(CloseLoad.y,       V5.theta) 
      annotation (points=[29,38; 38,38; 38,18],    style(color=3));
    connect(OpenRelief.y,       V4.theta) annotation (points=[-73,84; -68,84;
          -68,-48; -28,-48; -28,-60],              style(color=3));
    connect(CloseValves.y,       V3.theta) 
      annotation (points=[-75, -2; -28, -2; -28, -20], style(color=3));
    connect(CloseValves.y,       V2.theta) annotation (points=[-75, -2; -42,
          -2; -42, 54; -28, 54; -28, 44], style(color=3));
    connect(OpenRelief.y, V1.theta) annotation (points=[-73,84; -40,84; -40,76],
        style(color=74, rgbcolor={0,0,127}));
    connect(V2.port_a, SourceP1.port) annotation (points=[-39,36; -64,36; -64,
          41; -81.1,41], style(color=69, rgbcolor={0,127,255}));
    connect(V5.port_b, SinkP1.port) annotation (points=[49,10; 59.2,10], style(
          color=69, rgbcolor={0,127,255}));
    connect(V4.port_b, SinkP3.port) annotation (points=[-17,-68; -2.8,-68],
        style(color=69, rgbcolor={0,127,255}));
    connect(V5.port_a, volume.port) annotation (points=[27,10; 12.5,10; 12.5,9; 
          -2,9], style(color=69, rgbcolor={0,127,255}));
    connect(V3.port_b, volume.port) annotation (points=[-17,-28; -2,-28; -2,9],
        style(color=69, rgbcolor={0,127,255}));
    connect(volume.port, V2.port_b) annotation (points=[-2,9; -2,36; -17,36],
        style(color=69, rgbcolor={0,127,255}));
    connect(V3.port_a, SourceP2.port) annotation (points=[-39,-28; -60,-28; -60,
          -40; -83.2,-40], style(color=69, rgbcolor={0,127,255}));
    connect(V4.port_a, SourceP2.port) annotation (points=[-39,-68; -60,-68; -60,
          -40; -83.2,-40], style(color=69, rgbcolor={0,127,255}));
    connect(V1.port_b, SinkP2.port) annotation (points=[-29,68; -16.9,68],
        style(color=69, rgbcolor={0,127,255}));
    connect(V1.port_a, SourceP1.port) annotation (points=[-51,68; -64,68; -64,
          41; -81.1,41], style(color=69, rgbcolor={0,127,255}));
  end ValveReversingFlows;
  
end ThermodynamicTutorial;
