model CycleDriver "Drive cycle driver" 
  extends VehicleSystems.Icons.Driver;
  extends VehicleSystems.Interfaces.Subsystems.Driver(
                                           redeclare connector DriverBus = 
        LocalDriverBus);
  replaceable record CycleData = Types.DriveCycle annotation (
      choicesAllMatching =                                                      true);
  Modelica.Mechanics.Translational.Position brake_actuator 
    annotation (extent=[50,-10; 70,10]);
  Modelica.Mechanics.Translational.Position accel_actuator 
    annotation (extent=[50,-70; 70,-50]);
  Modelica.Mechanics.Rotational.Position steering_wheel 
    annotation (extent=[50,50; 70,70]);
  Modelica.Mechanics.Rotational.Fixed ground annotation (extent=[50,30; 70,50]);
  Modelica.Blocks.Sources.Constant steering_angle(k=0) 
    annotation (extent=[12,50; 32,70]);
  Modelica.Blocks.Sources.CombiTimeTable speed_setpoint(table=speed_table) 
                         annotation (extent=[-100,30; -80,50]);
  block Controller 
    parameter Real T=1;
    parameter Real k=0.5;
    Modelica.Blocks.Interfaces.RealInput speed_command 
      annotation (extent=[-120,-10; -100,10]);
    Modelica.Blocks.Interfaces.RealInput vehicle_speed 
      annotation (extent=[-10,-120; 10,-100], rotation=90);
    annotation (Diagram, Icon(Rectangle(extent=[-100,100; 100,-100], style(
            color=3,
            rgbcolor={0,0,255},
            fillColor=7,
            rgbfillColor={255,255,255})), Text(
          extent=[-100,40; 100,-40],
          style(color=3, rgbcolor={0,0,255}),
          string="%name")));
    Modelica.Blocks.Interfaces.RealOutput brake_pedal 
      annotation (extent=[100,50; 120,70]);
    Modelica.Blocks.Interfaces.RealOutput accel_pedal 
      annotation (extent=[100,-70; 120,-50]);
  protected 
    Real u "Error (input signal)";
    Real x "Internal state for PI";
    Real y "PI command";
    Real a "Acceleration command";
    Real b "Brake command";
  equation 
    u =speed_command-vehicle_speed;
    der(x) = u/T;
  //  y = k*(x+u);
    y = k*u;
    a = max(0,y);
    b = if speed_command          >=1 then min(0,y) else -1.0;
    accel_pedal           = min(max(a,0),1);
    brake_pedal           = min(max(-b,0),1);
  end Controller;
  output Types.KPH actual_speed=driver_bus.vehicle_speed;
  output Types.KPH desired_speed=speed_setpoint.y[1];
protected 
  final parameter CycleData cycle;
  final parameter Types.SpeedTrace speed_trace = cycle.speed_trace;
  final parameter Real speed_table[:,:] = [speed_trace.t, speed_trace.v];
  connector LocalDriverBus 
    extends VehicleSystems.Interfaces.Subsystems.MinimalDriverBus;
    Modelica.Blocks.Interfaces.RealSignal vehicle_speed(
                                                      redeclare type SignalType
        = Types.KPH);
    Modelica.Blocks.Interfaces.IntegerSignal requested_gear;
  end LocalDriverBus;
  Integer cur_gear_idx;
public 
  Modelica.Blocks.Interfaces.Adaptors.ReceiveReal vehicle_speed 
    annotation (extent=[50,-100; 70,-80],rotation=180);
  Controller actuator_commands annotation (extent=[-40,-40; -20,-20]);
  Modelica.Blocks.Interfaces.Adaptors.SendInteger send_gear 
    annotation (extent=[50,-40; 70,-20]);
algorithm 
  when initial() then
    cur_gear_idx := 1;
    while cycle.gear_schedule.times[cur_gear_idx]<time loop
      cur_gear_idx := cur_gear_idx + 1;
    end while;
  end when;
  when cur_gear_idx<size(cycle.gear_schedule.times,1) and time>=cycle.gear_schedule.times[cur_gear_idx+1] then
    cur_gear_idx := pre(cur_gear_idx)+1;
  end when;
equation 
  send_gear.u = cycle.gear_schedule.gear[cur_gear_idx];
  connect(brake_actuator.flange_b, driver_bus.brake_pedal) 
    annotation (points=[70,0; 100,0], style(color=58, rgbcolor={0,191,0}));
  connect(accel_actuator.flange_b, driver_bus.accelerator_pedal) annotation (
      points=[70,-60; 80,-60; 80,0; 100,0],style(color=58, rgbcolor={0,191,0}));
  connect(steering_angle.y,steering_wheel.phi_ref) 
    annotation (points=[33,60; 48,60], style(color=3, rgbcolor={0,0,255}));
  connect(steering_wheel.bearing,ground. flange_b) 
    annotation (points=[60,50; 60,40], style(color=0, rgbcolor={0,0,0}));
  connect(steering_wheel.flange_b, driver_bus.steering_wheel) annotation (
      points=[70,60; 80,60; 80,0; 100,0], style(color=0, rgbcolor={0,0,0}));
  annotation (Diagram);
  connect(vehicle_speed.fromBus, driver_bus.vehicle_speed) annotation (points=[71,
        -90; 100,-90; 100,0], style(
      color=77,
      rgbcolor={127,0,255},
      fillColor=10,
      rgbfillColor={135,135,135},
      fillPattern=1));
  connect(vehicle_speed.y,       actuator_commands.vehicle_speed) annotation (
     points=[49,-90; -30,-90; -30,-41], style(color=3, rgbcolor={0,0,255}));
  connect(speed_setpoint.y[1],    actuator_commands.speed_command) 
    annotation (points=[-79,40; -60,40; -60,-30; -41,-30], style(color=3,
        rgbcolor={0,0,255}));
  connect(actuator_commands.brake_pedal,brake_actuator.s_ref)   annotation (
      points=[-19,-24; 0,-24; 0,0; 48,0], style(color=3, rgbcolor={0,0,255}));
  connect(actuator_commands.accel_pedal,accel_actuator.s_ref)   annotation (
      points=[-19,-36; 0,-36; 0,-60; 48,-60], style(color=3, rgbcolor={0,0,
          255}));
  connect(send_gear.toBus, driver_bus.requested_gear) annotation (points=[71,
        -30; 100,-30; 100,0], style(color=77, rgbcolor={127,0,255}));
end CycleDriver;
