package Gearboxes 
  package BaseClasses 
    partial model VariableRatioGearbox 
      "Variable ratio gearboxBasic step-ratio transmission (no clutches)" 
      extends Interfaces.Gearbox;
      parameter Modelica.SIunits.Time shift_time = 0.2;
    protected 
      Modelica.SIunits.Time last_time;
      Real next_ratio "final ratio for current profile";
      Boolean shift;
      Real ratio(start=1) "Gear ratio";
      Real per;
      Real coef[4] "Coefficients for the ratio profile";
    initial equation 
      ratio = next_ratio;
    algorithm 
      when initial() then
        last_time := time-1.0;
        coef :={0,0,0,next_ratio};
      end when;
      when pre(shift) then
        coef[4] := pre(ratio);
        coef[3] := 3*coef[1]*pre(per)^2+2*coef[2]*pre(per)+coef[3];
        coef[2] := 3*next_ratio-2*coef[3]-3*coef[4];
        coef[1] := next_ratio-coef[2]-coef[3]-coef[4];
        last_time := time;
      end when;
      when time>=last_time+shift_time then
        coef := {0, 0, 0, pre(ratio)};
      end when;
    equation 
      per =  min(1.0, (time-last_time)/shift_time);
      der(turbine_shaft.phi) = -ratio*der(output_shaft.phi);
      -ratio*turbine_shaft.tau = output_shaft.tau;
      der(ratio) = (3*coef[1]*per^2+2*coef[2]*per+coef[3])/shift_time;
      annotation (Documentation(info="<html>
<h2>Overview</h2>

<p>
This model is for a step ratio transmission.  What is interesting about this model is that <b>the clutches are not modeled</b>.  Instead, an effective gear ratio changes smoothly from one ratio to the next.  The smoothness itself is important.  If the gear ratio slewed linearly between the initial ratio and the final ratio then the transmission would cause problems for the index reduction process (because it would lead to step changes in the acceleration values).
</p>

<h2>Parameters</h2>

<p>
The ratio values for the 1st-nth gears are specified by the <tt>ratios</tt> parameter.  The time required to move from one gear ratio to the next is specified by the <tt>shift_time</tt> parameter.
</p>

<h2>Control</h2>

<p>
Since this model has no clutches, there is no need for any kind of clutch scheduling.  Instead, the controller simply requests a specific gear.  A value between <i>1</i> and <i>n</i> (where n is the number of values in the <tt>ratios</tt> parameter) indicates the transmission should smoothly change the effective gear ratio to the specific value in the <tt>ratios</tt> array.  A value of <i>0</i> indicates a neutral state and a value of <i>-1</i> indicates the reverse gear.  In the case of reverse, the same gear ratio is used as for gear <i>1</i> but the sign of the ratio is reversed.
</p>

<p>
Note that the formulation for the ratio transitions <b>can handle \"change of mind\" events</b> (<i>i.e.</i> the controller requests a new gear before the previous \"shift\" is complete).  In all cases, the effective gear ratio profile will always have a continuous first derivative (as requires for index reduction).
</p>
</html>
"),   Diagram);
    end VariableRatioGearbox;
    
    partial model VariableRatioTransmission 
      "Variable ratio transmissionBasic step-ratio transmission (no clutches)" 
      extends VehicleSystems.Interfaces.Subsystems.Transmission;
      parameter Modelica.SIunits.Time shift_time = 0.2;
    protected 
      Modelica.SIunits.Time last_time;
      Real next_ratio "final ratio for current profile";
      Boolean shift;
      Real ratio(start=1) "Gear ratio";
      Real per;
      Real coef[4] "Coefficients for the ratio profile";
    initial equation 
      ratio = next_ratio;
    algorithm 
      when initial() then
        last_time := time-1.0;
        coef :={0,0,0,next_ratio};
      end when;
      when pre(shift) then
        coef[4] := pre(ratio);
        coef[3] := 3*coef[1]*pre(per)^2+2*coef[2]*pre(per)+coef[3];
        coef[2] := 3*next_ratio-2*coef[3]-3*coef[4];
        coef[1] := next_ratio-coef[2]-coef[3]-coef[4];
        last_time := time;
      end when;
      when time>=last_time+shift_time then
        coef := {0, 0, 0, pre(ratio)};
      end when;
    equation 
      per =  min(1.0, (time-last_time)/shift_time);
      der(input_shaft.phi) = ratio*der(output_shaft.phi);
      -ratio*input_shaft.tau = output_shaft.tau;
      der(ratio) = (3*coef[1]*per^2+2*coef[2]*per+coef[3])/shift_time;
      annotation (Documentation(info="<html>
<h2>Overview</h2>

<p>
This model is for a step ratio transmission.  What is interesting about this model is that <b>the clutches are not modeled</b>.  Instead, an effective gear ratio changes smoothly from one ratio to the next.  The smoothness itself is important.  If the gear ratio slewed linearly between the initial ratio and the final ratio then the transmission would cause problems for the index reduction process (because it would lead to step changes in the acceleration values).
</p>

<h2>Parameters</h2>

<p>
The ratio values for the 1st-nth gears are specified by the <tt>ratios</tt> parameter.  The time required to move from one gear ratio to the next is specified by the <tt>shift_time</tt> parameter.
</p>

<h2>Control</h2>

<p>
Since this model has no clutches, there is no need for any kind of clutch scheduling.  Instead, the controller simply requests a specific gear.  A value between <i>1</i> and <i>n</i> (where n is the number of values in the <tt>ratios</tt> parameter) indicates the transmission should smoothly change the effective gear ratio to the specific value in the <tt>ratios</tt> array.  A value of <i>0</i> indicates a neutral state and a value of <i>-1</i> indicates the reverse gear.  In the case of reverse, the same gear ratio is used as for gear <i>1</i> but the sign of the ratio is reversed.
</p>

<p>
Note that the formulation for the ratio transitions <b>can handle \"change of mind\" events</b> (<i>i.e.</i> the controller requests a new gear before the previous \"shift\" is complete).  In all cases, the effective gear ratio profile will always have a continuous first derivative (as requires for index reduction).
</p>
</html>
"),   Diagram);
    end VariableRatioTransmission;
  end BaseClasses;
  
  model ClutchlessStepRatioGearbox "Clutchless step-ratio transmission" 
    parameter Real ratios[:] "Gear ratios (1st-nth gear)";
    extends BaseClasses.VariableRatioGearbox(redeclare connector 
        TransmissionBus = 
          GearNumberBus);
  protected 
    connector GearNumberBus 
      extends VehicleSystems.Icons.Bus;
      input Integer requested_gear_number;
    end GearNumberBus;
    
    function Ratio 
      input Integer i;
      input Real ratios[:];
      output Real R;
    algorithm 
      assert((i>=1 and i<=size(ratios,1)) or i==0 or i==-1,
             "Invalid gear ratio requested");
      if i>=1 and i<=size(ratios,1) then
        R := ratios[i];
      elseif i==0 then
        R := 0;
      else
        R := -ratios[1];
      end if;
    end Ratio;
  equation 
    shift = change(transmission_bus.requested_gear_number);
    next_ratio =  Ratio(transmission_bus.requested_gear_number,ratios);
  end ClutchlessStepRatioGearbox;
  
  model ContinuouslyVariableGearbox "CVT Transmission" 
    extends BaseClasses.VariableRatioGearbox;
  end ContinuouslyVariableGearbox;
end Gearboxes;
